var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var SuperBubbles1443113639239;
        (function (SuperBubbles1443113639239) {
            var SelectionManager = visuals.utility.SelectionManager;
            ;
            var Bubble = (function () {
                function Bubble() {
                }
                Bubble.converter = function (dataView, colors) {
                    var catDv = dataView.categorical;
                    var cat = catDv.categories[0];
                    var catValues = cat.values;
                    var values = catDv.values;
                    var dataPoints = [];
                    var formatStringProp = { objectName: 'general', propertyName: 'formatString' };
                    var categorySourceFormatString = visuals.valueFormatter.getFormatString(cat.source, formatStringProp);
                    for (var i = 0, len = catValues.length; i < len; i++) {
                        var formattedCategoryValue = visuals.valueFormatter.format(catValues[i], categorySourceFormatString);
                        dataPoints.push({
                            name: catValues[i],
                            value: values[0].values[i],
                            packageName: "main",
                            label: catValues[i],
                            color: colors.getColorByIndex(i).value,
                            selector: visuals.SelectionId.createWithId(cat.identity[i]),
                            tooltipInfo: [{
                                displayName: formattedCategoryValue,
                                value: values[0].values[i],
                            }]
                        });
                    }
                    return dataPoints;
                };
                Bubble.prototype.init = function (options) {
                    var element = options.element;
                    this.selectionManager = new SelectionManager({ hostServices: options.host });
                    this.svg = d3.select(element.get(0)).append('svg').classed(Bubble.VisualClassName, true);
                    this.colors = options.style.colorPalette.dataColors;
                };
                Bubble.prototype.update = function (options) {
                    var _this = this;
                    if (!options.dataViews || !options.dataViews[0])
                        return; // or clear the view, display an error, etc.
                    if (this.mainGroupElement !== undefined)
                        this.mainGroupElement.remove();
                    this.mainGroupElement = this.svg.append('g');
                    var dataView = this.dataView = options.dataViews[0];
                    var dataPoints = Bubble.converter(dataView, this.colors);
                    var viewport = options.viewport;
                    this.svg.attr({
                        'height': viewport.height,
                        'width': viewport.width
                    }).on('click', function () { return _this.selectionManager.clear().then(function () { return nodeCircle.style('opacity', 1); }); });
                    var width = viewport.width;
                    var height = viewport.height;
                    var mydata = { name: "Parent", children: dataPoints };
                    var mainGroup = this.mainGroupElement;
                    this.bubble = d3.layout.pack().sort(null).size([width, height]).padding(1.5);
                    var data = this.bubble.nodes(mydata);
                    var selectionManager = this.selectionManager;
                    if (width > 10 && height > 10) {
                        var selection = mainGroup.selectAll(".bubbleCircles").data(data.filter(function (d) {
                            return !d.children;
                        }));
                        selection.enter().append("g").classed("bubbleCircles", true).attr("cursor", "pointer").attr("transform", function (d) {
                            return "translate(" + d.x + "," + d.y + ")";
                        });
                        var nodeCircle = selection.append("circle").attr('fill', function (d) { return d.color; });
                        nodeCircle.on('click', function (d) {
                            var _this = this;
                            selectionManager.select(d.selector).then(function (ids) {
                                if (ids.length > 0) {
                                    nodeCircle.style('opacity', 0.5);
                                    d3.select(_this).style('opacity', 1);
                                }
                                else {
                                    nodeCircle.style('opacity', 1);
                                }
                            });
                            d3.event.stopPropagation();
                        });
                        nodeCircle.attr("r", 0).transition().duration(1200).attr("r", function (d) {
                            if (d.name === "Parent") {
                                return 0;
                            }
                            else {
                                return d.r;
                            }
                            ;
                        });
                        var nodeText = selection.append("text").attr("dy", ".3em").style("text-anchor", "middle").attr("font-size", function (d) {
                            return d.r / 3 > 5 ? d.r / 3 : 5;
                        }).text(function (d) {
                            if (d.name === "Parent") {
                                return "";
                            }
                            else {
                                var substring = d.name.substring(0, d.r / (d.r / 3) * 3);
                                return substring.length < d.name.length ? substring + "." : substring;
                            }
                            ;
                        }).style("fill", this.getLabelFill(this.dataView).solid.color);
                        nodeText.on('click', function (d) {
                            selectionManager.select(d.selector).then(function (ids) {
                                if (ids.length > 0) {
                                    nodeCircle.style("opacity", function (node) {
                                        if (d == node)
                                            return 1;
                                        else
                                            return 0.5;
                                    });
                                }
                                else {
                                    nodeCircle.style('opacity', 1);
                                }
                            });
                            d3.event.stopPropagation();
                        });
                        nodeText.attr("fill", "#fff").transition().duration(1000).attr("fill", "#000");
                        visuals.TooltipManager.addTooltip(nodeCircle, function (tooltipEvent) { return tooltipEvent.data.tooltipInfo; });
                    }
                };
                // This extracts fill color of the label from the DataView
                Bubble.prototype.getLabelFill = function (dataView) {
                    if (dataView && dataView.metadata.objects) {
                        var label = dataView.metadata.objects['label'];
                        if (label) {
                            return label['fill'];
                        }
                    }
                    return { solid: { color: '#ffffff' } };
                };
                // This function retruns the values to be displayed in the property pane for each object.
                // Usually it is a bind pass of what the property pane gave you, but sometimes you may want to do
                // validation and return other values/defaults
                Bubble.prototype.enumerateObjectInstances = function (options) {
                    var instances = [];
                    switch (options.objectName) {
                        case 'label':
                            var label = {
                                objectName: 'label',
                                displayName: 'Label',
                                selector: null,
                                properties: {
                                    fill: this.getLabelFill(this.dataView)
                                }
                            };
                            instances.push(label);
                            break;
                    }
                    return instances;
                };
                Bubble.capabilities = {
                    dataRoles: [
                        {
                            name: 'Category',
                            kind: powerbi.VisualDataRoleKind.Grouping,
                        },
                        {
                            name: 'Y',
                            kind: powerbi.VisualDataRoleKind.Measure,
                            displayName: powerbi.data.createDisplayNameGetter('Role_DisplayName_Values'),
                        },
                    ],
                    dataViewMappings: [{
                        categories: {
                            for: { in: 'Category' },
                            dataReductionAlgorithm: { top: {} }
                        },
                        values: {
                            select: [{ bind: { to: 'Y' } }]
                        },
                    }],
                    objects: {
                        general: {
                            displayName: powerbi.data.createDisplayNameGetter('Visual_General'),
                            properties: {
                                formatString: {
                                    type: { formatting: { formatString: true } },
                                },
                            },
                        },
                        label: {
                            displayName: 'Label',
                            properties: {
                                fill: {
                                    displayName: 'Fill',
                                    type: { fill: { solid: { color: true } } }
                                }
                            }
                        },
                        outerLine: {
                            displayName: 'Outer line',
                            properties: {
                                show: {
                                    displayName: 'Show',
                                    type: { bool: true }
                                },
                                thickness: {
                                    displayName: 'Thickness',
                                    type: { numeric: true }
                                }
                            }
                        }
                    }
                };
                Bubble.VisualClassName = 'Bubble';
                return Bubble;
            })();
            SuperBubbles1443113639239.Bubble = Bubble;
            ;
        })(SuperBubbles1443113639239 = visuals.SuperBubbles1443113639239 || (visuals.SuperBubbles1443113639239 = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));
;
var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var plugins;
        (function (plugins) {
            plugins.SuperBubbles1443113639239 = {
                name: 'SuperBubbles1443113639239',
                class: 'SuperBubbles1443113639239',
                capabilities: powerbi.visuals.SuperBubbles1443113639239.Bubble.capabilities,
                custom: true,
                create: function () { return new powerbi.visuals.SuperBubbles1443113639239.Bubble(); }
            };
        })(plugins = visuals.plugins || (visuals.plugins = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));
